﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ChartAnnotation.Classes;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ChartView;

namespace ChartAnnotation
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private List<ViewModel> datafeed = new List<ViewModel>();
        private ScaleTransform _st = new ScaleTransform();
        private Scaling _scaling = new Scaling();

        public MainWindow()
        {
            InitializeComponent();

            //set up bindings for the scaling values, so we add the ability for autoscaling of the signal objects
            Binding bindingX = new Binding("ScaleX");
            bindingX.Source = _scaling;

            Binding bindingY = new Binding("ScaleY");
            bindingY.Source = _scaling;

            BindingOperations.SetBinding(_st, ScaleTransform.ScaleXProperty, bindingX);
            BindingOperations.SetBinding(_st, ScaleTransform.ScaleYProperty, bindingY);


            //generate some random data
            Random rd = new Random();
            for (int i = 0; i < 30; i++)
            {
                ViewModel vm = new ViewModel();
                vm.High = rd.Next(8 + i, 10 + i);
                vm.Low = rd.Next(2 + i, 4 + i);
                vm.Close = rd.Next(4 + i, 8 + i);
                vm.Open = rd.Next(4 + i, 8 + i);

                int random = rd.Next(1, 10);
                if (random < 3)
                {
                    vm.Direction = 1;
                    vm.Signal = vm.Low;
                }
                else if (random > 7)
                {
                    vm.Direction = -1;
                    vm.Signal = vm.High;
                }

                vm.Category = i;
                datafeed.Add(vm);
            }


            //create our candlestick series
            CandlestickSeries newSerie = new CandlestickSeries();
            newSerie.OpenBinding = new PropertyNameDataPointBinding("Open");
            newSerie.HighBinding = new PropertyNameDataPointBinding("High");
            newSerie.CloseBinding = new PropertyNameDataPointBinding("Close");
            newSerie.LowBinding = new PropertyNameDataPointBinding("Low");
            newSerie.CategoryBinding = new PropertyNameDataPointBinding("Category");
            this.chart.Series.Add(newSerie);
            newSerie.ItemsSource = datafeed;


            //for use drawing object instead of path uncomment

            //FrameworkElementFactory ellipse = new FrameworkElementFactory(typeof(Ellipse));
            //ellipse.SetValue(Ellipse.FillProperty, Brushes.Red);
            //ellipse.SetValue(Ellipse.WidthProperty, 10d);
            //ellipse.SetValue(Ellipse.HeightProperty, 10d);

            //add the signal series for sell signals
            LineSeries sell = new LineSeries();
            sell.Stroke = Brushes.Transparent;
            sell.ValueBinding = new PropertyNameDataPointBinding("Signal");
            sell.CategoryBinding = new PropertyNameDataPointBinding("Category");
            sell.PointTemplate = new DataTemplate()
                {
                    VisualTree = GeometryCode(-1)

                    //for use drawing object instead of path uncomment
                    //VisualTree = ellipse
                };
            sell.ItemsSource = datafeed.Where(x => x.Direction < 0);
            chart.Series.Add(sell);


            //add the signal series for buy signals
            LineSeries buy = new LineSeries();
            buy.Stroke = Brushes.Transparent;
            buy.ValueBinding = new PropertyNameDataPointBinding("Signal");
            buy.CategoryBinding = new PropertyNameDataPointBinding("Category");
            buy.PointTemplate = new DataTemplate()
                {
                    VisualTree = GeometryCode(1)

                    //for use drawing object instead of path uncomment
                    //VisualTree = ellipse
                };
            buy.ItemsSource = datafeed.Where(x => x.Direction > 0);
            chart.Series.Add(buy);
        }

        private void chart_Loaded(object sender, RoutedEventArgs e)
        {
            PanZoomBar horizontalZoomBar = this.chart.HorizontalAxis.ChildrenOfType<PanZoomBar>().FirstOrDefault();
            if (horizontalZoomBar != null)
            {
                horizontalZoomBar.SelectionChanged += new Telerik.Windows.RadRoutedEventHandler(horizontalZoomBar_SelectionChanged);
            }
            Rescale();
        }

        void horizontalZoomBar_SelectionChanged(object sender, Telerik.Windows.RadRoutedEventArgs e)
        {
            Rescale();
        }


        //rescale automatically to the data, visible on the screen
        void Rescale()
        {
            double pixelsPerElem = this.chart.PlotAreaClip.Width * this.chart.Zoom.Width / datafeed.Count();
            int firstElem = (int)(-this.chart.PanOffset.X / pixelsPerElem);
            int lastElem = firstElem + (int)(this.chart.PlotAreaClip.Width / pixelsPerElem);

            int take = Math.Max(lastElem - firstElem, 1);
            var visibleRange = datafeed.Skip(firstElem).Take(take + 1);
            var yAxisVisibleMinValue = (double)visibleRange.Select(x => x.Low).Min();
            var yAxisVisibleMaxValue = (double)visibleRange.Select(x => x.High).Max();

            var yAxis = this.chart.VerticalAxis as LinearAxis;
            yAxis.Minimum = yAxisVisibleMinValue;
            yAxis.Maximum = yAxisVisibleMaxValue;

            //reduce the category labels amount in case of large set of data
            if (take > 10)
            {
                var xAxis = this.chart.HorizontalAxis as CategoricalAxis;
                xAxis.LabelInterval = take / 10;
            }


            //adjust the scaling of the signal objects
            int offset = (int)(this.chart.Zoom.Width * (4) + 10);
            _scaling.ScaleX = offset;
            _scaling.ScaleY = offset;
        }



        //create signal object from path code
        private FrameworkElementFactory GeometryCode(int direction)
        {
            FrameworkElementFactory path = new FrameworkElementFactory(typeof(Path));
            Brush clr = Brushes.Yellow;
            TransformGroup transformGroup = new TransformGroup();

            string code = "M0,1 L1,1 0.5,0Z";//triangle path code
            if (direction > 0)
            {
                clr = Brushes.ForestGreen;
            }
            else if (direction < 0)
            {
                clr = Brushes.Firebrick;
                RotateTransform rotation = new RotateTransform(180);
                transformGroup.Children.Add(rotation);
            }
            transformGroup.Children.Add(_st);

            path.SetValue(Path.DataProperty, Geometry.Parse(code));
            path.SetValue(Path.FillProperty, clr);
            path.SetValue(Path.RenderTransformProperty, transformGroup);

            //center the object horizontally when scaling
            path.SetValue(Path.RenderTransformOriginProperty, new Point(0.5, 0));

            return path;
        }


    }
}
